#include <IRremote.h>
#include <Stepper.h>
#include <Servo.h>

IRrecv irrecv(2);
decode_results results;

const int stepsPerRevolution = 2048; // Set the number of steps per revolution
Stepper myStepper(stepsPerRevolution, 8, 10, 9, 11); // Configure the appropriate pins
Servo myServo;
int servoPosition = 90;  // Set initial servo position to 90 degrees
int lightSensorPin = A0;

const int rgbRedPin = 4;
const int rgbGreenPin = 5;
const int rgbBluePin = 6;
const int buzzerPin = 12;
const int ledPin = 13;

bool lightSensorActive = true; // Variable to control the light sensor

void setup() {
  irrecv.enableIRIn();
  myStepper.setSpeed(10); // Set the speed of the stepper motor to 10
  myServo.attach(7);
  pinMode(lightSensorPin, INPUT);
  pinMode(rgbRedPin, OUTPUT);
  pinMode(rgbGreenPin, OUTPUT);
  pinMode(rgbBluePin, OUTPUT);
  pinMode(buzzerPin, OUTPUT);
  pinMode(ledPin, OUTPUT);

  Serial.begin(9600); // Initialize Serial communication with a baud rate of 9600

  // Set the initial RGB LED state to blue
  setRGBColor(0, 0, 255);  // Blue
}

void loop() {
  // Read the light sensor value if it's active
  if (lightSensorActive) {
    int lightValue = analogRead(lightSensorPin);

    // Check the light level and control the light accordingly
    if (lightValue < 200) {
      digitalWrite(ledPin, HIGH); // Turn on the LED
    } else {
      digitalWrite(ledPin, LOW); // Turn off the LED
    }
  }

  if (irrecv.decode(&results)) {
    // Print the received IR code to the Serial Monitor
    Serial.println(results.value, HEX);

    // Button 1: Control the bin
    if (results.value == 0xFF30CF) {
      // Set RGB LED to the red state
      setRGBColor(255, 0, 0);
      digitalWrite(buzzerPin, HIGH);
      delay(300);
      digitalWrite(buzzerPin, LOW);
      
      // Move the hand back (180 degrees)
      myServo.write(180);

      // Move the bin forward
      for (int i = 0; i < 800; i++) {
        myStepper.step(-1);
        delay(5); // Adjust the delay for your desired speed
      }

      delay(2000); // Wait 2 seconds

      // Move the bin back
      for (int i = 0; i < 800; i++) {
        myStepper.step(1);
        delay(5); // Adjust the delay for your desired speed
      }

      setRGBColor(0, 255, 0);
      digitalWrite(buzzerPin, HIGH);
      delay(300);
      digitalWrite(buzzerPin, LOW);
      // Move the hand back to the initial position (90 degrees)
      myServo.write(90);
      delay(3000);
      // Set the initial RGB LED state to blue
      setRGBColor(0, 0, 255);  // Blue
    }
    
    // Button 2: Flash RGB LED and wave hand
    else if (results.value == 0xFF18E7) {
      flashLEDAndWaveHand();
    }
    
    // Button 3: Turn on buzzer and LED for 5 seconds
    else if (results.value == 0xFF7A85) {
      // Set RGB LED to the red state
      setRGBColor(255, 0, 0);
      digitalWrite(ledPin, HIGH);
      for (int i = 0; i < 5; i++) {
      digitalWrite(buzzerPin, HIGH);
      delay(500);
      digitalWrite(buzzerPin, LOW);
      delay(500);
      }
      digitalWrite(ledPin, LOW);
      delay(3000);
      // Set the initial RGB LED state to blue
      setRGBColor(0, 0, 255);  // Blue
    }

    irrecv.resume();
  }
}

// Function to set the RGB LED color
void setRGBColor(int red, int green, int blue) {
  analogWrite(rgbRedPin, red);
  analogWrite(rgbGreenPin, green);
  analogWrite(rgbBluePin, blue);
}

// Function to flash RGB LED with different colors and wave the hand
void flashLEDAndWaveHand() {
  // Flash RGB LED with different colors for 4 oscillations
  for (int i = 0; i < 4; i++) {
    setRGBColor(255, 0, 0);  // Red
    myServo.write(120); // Set servo to 120 degrees
    delay(1000);
    setRGBColor(0, 0, 255);  // Blue
    myServo.write(90); // Set servo to 90 degrees
    delay(1000);
    setRGBColor(0, 255, 0);  // Green
    myServo.write(60); // Set servo to 60 degrees
    delay(1000);
    setRGBColor(0, 0, 255);  // Blue
    myServo.write(90); // Set servo to 90 degrees
    delay(1000);
  }
}
